import { render, screen, waitFor } from "@testing-library/react";
import { MemoryRouter } from "react-router-dom";
import App from "../../App";
import mockFetch from "./mockFetch";

import { ErrorBoundary, withErrorBoundary } from "react-error-boundary";
import ErrorFallback from "../ErrorFallback";
import {
  ErrorFromValidation,
  fetchWithErrorHandling,
} from "../../backend/validation";
import { ShopItemResource } from "../../Resources";
import { shopItems1 } from "./data";
import ShopItem from "../ShopItem";

// 1000 is the default anyway
const MAX_TIMEOUT_FOR_FETCH_TESTS = Number.parseInt(
  process.env.MAX_TIMEOUT_FOR_FETCH_TESTS || "1000"
);

function waitForLonger(callback: () => void | Promise<void>) {
  return waitFor(callback, { timeout: MAX_TIMEOUT_FOR_FETCH_TESTS });
}

const AppWithErrorBoundary = withErrorBoundary(App, {
  onError: (err, info) => {
    throw err;
  },
  fallback: <div />,
});

test("Shopper", async () => {
  const orgError = console.error;
  mockFetch();
  try {
    console.error = () => {};

    render(
      <MemoryRouter initialEntries={["/"]}>
        <AppWithErrorBoundary />
      </MemoryRouter>
    );
  } finally {
    console.error = orgError;
  }

  await waitForLonger(() => {
    const title = screen.getAllByText(/Store/i);
    expect(title.length).toBeGreaterThanOrEqual(2);
  });

  //const title = await screen.findAllByText(/Channel [12]/i);
  //expect(title.length).toBeGreaterThanOrEqual(2);
});

// Testen Sie mal folgende Routen (in initialEntries):
// /shoplist/123456"
test("shopList /shoplist/123456", async () => {
  const orgError = console.error;
  mockFetch();
  try {
    console.error = () => {};

    render(
      <MemoryRouter initialEntries={["/shoplist/123456"]}>
        <AppWithErrorBoundary />
      </MemoryRouter>
    );
  } finally {
    console.error = orgError;
  }

  await waitForLonger(() => {
    const title = screen.getAllByText(/Store/i);
    expect(title.length).toBeGreaterThanOrEqual(2);
  });
});

// /shopitem/51"
test("shopList /shopitem/51", async () => {
  const orgError = console.error;
  mockFetch();
  try {
    console.error = () => {};

    render(
      <MemoryRouter initialEntries={["/shopitem/51"]}>
        <AppWithErrorBoundary />
      </MemoryRouter>
    );
  } finally {
    console.error = orgError;
  }

  await waitForLonger(() => {
    const title = screen.getAllByText(/Store/i);
    expect(title.length).toBeGreaterThanOrEqual(2);
  });
});

test("Shopitem test1", async () => {
  jest.spyOn(global, "fetch").mockImplementation(() =>
    Promise.resolve({
      ok: true,
      json: () => Promise.resolve(shopItems1),
    } as Response)
  );

  render(<ShopItem shopItem={shopItems1[0] as ShopItemResource} />);

  await waitFor(
    () => {
      expect(screen.getByText(/Anzahl/i)).toBeInTheDocument();
      expect(screen.getByText(/Bemerkung/i)).toBeInTheDocument();
    },
    { timeout: 3000 }
  );
});

test("shopList /admin", async () => {
  const orgError = console.error;
  mockFetch();
  try {
    console.error = () => {};

    render(
      <MemoryRouter initialEntries={["/admin"]}>
        <AppWithErrorBoundary />
      </MemoryRouter>
    );
  } finally {
    console.error = orgError;
  }

  await waitForLonger(() => {
    expect(screen.getByText(/PageAdmin/i)).toBeInTheDocument();
  });
});

test("shopList /pageprefs", async () => {
  const orgError = console.error;
  mockFetch();
  try {
    console.error = () => {};

    render(
      <MemoryRouter initialEntries={["/prefs"]}>
        <AppWithErrorBoundary />
      </MemoryRouter>
    );
  } finally {
    console.error = orgError;
  }

  await waitForLonger(() => {
    expect(screen.getByText(/PagePrefs/i)).toBeInTheDocument();
  });
});

test("shopItem ID falsch /shopitem/abc", async () => {
  const orgError = console.error;
  mockFetch();
  try {
    console.error = () => {};

    render(
      <MemoryRouter initialEntries={["/shopitem/abc"]}>
        <AppWithErrorBoundary />
      </MemoryRouter>
    );
  } finally {
    console.error = orgError;
  }
});

test("shopItem LoadingIndicator", async () => {
  const orgError = console.error;
  mockFetch();
  try {
    console.error = () => {};

    render(
      <MemoryRouter initialEntries={["/prefs"]}>
        <AppWithErrorBoundary />
      </MemoryRouter>
    );
  } finally {
    console.error = orgError;
  }
  
  await waitForLonger(() => {
    expect(screen.getByText(/PagePrefs/i)).toBeInTheDocument();
  });
});

